/*
    Copyright (C) 2002 Timo Sirainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "common.h"
#include "network.h"
#include "lib-signals.h"
#include "restrict-access.h"
#include "auth.h"
#include "cookie.h"
#include "login-connection.h"
#include "userinfo.h"

#include <syslog.h>

void master_init(void);
void master_deinit(void);

IOLoop ioloop;
static IO io_listen;

static void sig_quit(int signo)
{
	io_loop_stop(ioloop);
}

static void auth_accept(void *user_data, int listen_fd, IO io)
{
	int fd;

	fd = net_accept(listen_fd, NULL, NULL);
	if (fd != -1)
		(void)login_connection_create(fd);
}

static void main_init(void)
{
	lib_init_signals(sig_quit);

	/* open the syslog immediately so chroot() won't break logging */
	openlog("imap-auth", LOG_NDELAY, LOG_MAIL);

	i_set_panic_handler(i_syslog_panic_handler);
	i_set_fatal_handler(i_syslog_fatal_handler);
        i_set_error_handler(i_syslog_error_handler);
        i_set_warning_handler(i_syslog_warning_handler);

	restrict_access_by_env();

	auth_init();
	cookies_init();
	login_connections_init();
	master_init();
	userinfo_init();

	io_listen = io_add(LOGIN_LISTEN_FD, IO_READ, auth_accept, NULL);
}

static void main_deinit(void)
{
        if (lib_signal_kill != 0)
		i_warning("Killed with signal %d", lib_signal_kill);

	io_remove(io_listen);

	userinfo_deinit();
	master_deinit();
	login_connections_deinit();
	cookies_deinit();
	auth_deinit();
}

int main(int argc, char *argv[])
{
	/* NOTE: we start rooted, so keep the code minimal until
	   restrict_access_by_env() is called */
	lib_init();
	ioloop = io_loop_create();

	main_init();
        io_loop_run(ioloop);
	main_deinit();

	//io_loop_destroy(ioloop);
	lib_deinit();

        return 0;
}
