/*
 fdpass.c - FD passing, based on an example by

    Copyright (C) 2001 Jakub Vlasek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef __sun__
#  define _XPG4_2
#endif

#include "lib.h"

#ifdef __sun__
#  define _XPG4_2
#endif
#include "network.h"

#include <sys/un.h>
#include <sys/uio.h>

#if defined (__sun__) && !defined(CMSG_SPACE)
#define CMSG_ALIGN(len) \
	(((len) + sizeof (size_t) - 1) & ~(sizeof (size_t) - 1))
#define CMSG_SPACE(len) \
	(CMSG_ALIGN (len) + CMSG_ALIGN (sizeof (struct cmsghdr)))
#define CMSG_LEN(len) \
	(CMSG_ALIGN (sizeof (struct cmsghdr)) + (len))
#endif

#if defined (__sun__) && !defined(CMSG_SPACE)
#define CMSG_ALIGN(len) \
	(((len) + sizeof (size_t) - 1) & ~(sizeof (size_t) - 1))
#define CMSG_SPACE(len) \
	(CMSG_ALIGN (len) + CMSG_ALIGN (sizeof (struct cmsghdr)))
#define CMSG_LEN(len) \
	(CMSG_ALIGN (sizeof (struct cmsghdr)) + (len))
#endif

int fd_send(int handle, int send_fd, const void *data, int size)
{
        struct msghdr msg;
        struct iovec iov;
        struct cmsghdr *cmsg;
	int *fdptr;
        char buf[CMSG_SPACE(sizeof(int))];

	memset(&msg, 0, sizeof (struct msghdr));

        iov.iov_base = (void *) data;
        iov.iov_len = size;
        msg.msg_control = buf;
        msg.msg_controllen = sizeof(buf);
        msg.msg_iov = &iov;
	msg.msg_iovlen = 1;

        cmsg = CMSG_FIRSTHDR(&msg);
        cmsg->cmsg_level = SOL_SOCKET;
        cmsg->cmsg_type = SCM_RIGHTS;
	cmsg->cmsg_len = CMSG_LEN(sizeof(int));
	fdptr = (int *) CMSG_DATA(cmsg);
	*fdptr = send_fd;
	return sendmsg(handle, &msg, 0);
}

int fd_read(int handle, void *data, int size, int *fd)
{
	struct msghdr msg;
	struct iovec iov;
	struct cmsghdr *cmsg;
	int ret;
	char buf[CMSG_SPACE(sizeof(int))];

	memset(&msg, 0, sizeof (struct msghdr));

	msg.msg_control = buf;
	msg.msg_controllen = sizeof(buf);
	msg.msg_iov = &iov;
	msg.msg_iovlen = 1;
	cmsg = CMSG_FIRSTHDR(&msg);
	cmsg->cmsg_level = SOL_SOCKET;
	cmsg->cmsg_type = SCM_RIGHTS;
	iov.iov_base = (void *) data;
	iov.iov_len = size;

	ret = recvmsg(handle, &msg, 0);
	*fd = *(int *) CMSG_DATA(cmsg);
	return ret;
}
