#ifndef __TREE_H
#define __TREE_H

/* Returns TRUE -1..1 */
typedef int (*TreeCompareFunc) (const void *p1, const void *p2);
/* Returns FALSE if it wants to stop. */
typedef int (*TreeTraverseFunc) (const void *key, const void *value,
				 void *user_data);

typedef struct _Tree Tree;
typedef struct _TreeNode TreeNode;

Tree *tree_create(Pool pool, TreeCompareFunc key_compare_func);
void tree_destroy(Tree *tree);

#ifdef POOL_CHECK_LEAKS
#  define tree_destroy_clean(tree) tree_destroy(tree)
#else
#  define tree_destroy_clean(tree)
#endif

void tree_ref(Tree *tree);
void tree_unref(Tree *tree);

void tree_insert(Tree *tree, const void *key, const void *value);
void tree_remove(Tree *tree, const void *key);

void *tree_lookup(Tree *tree, const void *key);
void tree_traverse(Tree *tree, TreeTraverseFunc traverse_func, void *data);

int tree_height(Tree *tree);
int tree_nodes_count(Tree *tree);

/* nodes: */
TreeNode *tree_search(Tree *tree, TreeCompareFunc func, const void *key);
TreeNode *tree_node_search(TreeNode *node, TreeCompareFunc compare,
			   const void *key);
int tree_node_traverse(TreeNode *node, TreeTraverseFunc traverse_func,
		       void *data);

int tree_node_count(TreeNode *node);
int tree_node_height(TreeNode *node);

#endif
